<?php
/**
 * @package money order payment module
 *
 * @package paymentMethod
 * @copyright Copyright 2003-2024 Zen Cart Development Team
 * @copyright Portions Copyright 2003 osCommerce
 * @license http://www.zen-cart.com/license/2_0.txt GNU Public License V2.0
 * @version $Id: Drbyte Sun Jan 7 21:30:21 2018 -0500 Modified in v1.5.6 $
 */
  class tcbpay {
    public $code;
    public $title;
    public $description;
    public $enabled;
    public $transaction_id;

    // class constructor
    function __construct() {
      global $order, $messageStack;

      $this->code = 'tcbpay';
      $this->title = MODULE_PAYMENT_TCBPAY_TEXT_TITLE;
      if (IS_ADMIN_FLAG === true) {
        if ($this->enabled) {
          if (MODULE_PAYMENT_TCBPAY_USERNAME == '' || MODULE_PAYMENT_TCBPAY_PASSWORD == '') {
            $this->title .=  '<span class="alert"> (Not Configured)</span>';
          }
          if (!function_exists('curl_init')) $messageStack->add_session('curl module not enabled.', 'error');
        }
      }
      $this->description = MODULE_PAYMENT_TCBPAY_TEXT_DESCRIPTION;
      $this->sort_order = defined('MODULE_PAYMENT_TCBPAY_SORT_ORDER') ? MODULE_PAYMENT_TCBPAY_SORT_ORDER : null;
      $this->enabled = (defined('MODULE_PAYMENT_TCBPAY_STATUS') && MODULE_PAYMENT_TCBPAY_STATUS == 'True');

      if (null === $this->sort_order) return false;

      $this->order_status = (int)DEFAULT_ORDERS_STATUS_ID;
      if ((int)MODULE_PAYMENT_TCBPAY_ORDER_STATUS_ID > 0) {
        $this->order_status = (int)MODULE_PAYMENT_TCBPAY_ORDER_STATUS_ID;
      }

      if (is_object($order)) $this->update_status();
    }

    // class methods
    function update_status() {
      global $order, $db;

      if ($this->enabled && (int)MODULE_PAYMENT_TCBPAY_ZONE > 0 && isset($order->billing['country']['id'])) {
        $check_flag = false;
        $check = $db->Execute("select zone_id from " . TABLE_ZONES_TO_GEO_ZONES . " where geo_zone_id = '" . MODULE_PAYMENT_TCBPAY_ZONE . "' and zone_country_id = '" . (int)$order->billing['country']['id'] . "' order by zone_id");
        while (!$check->EOF) {
          if ($check->fields['zone_id'] < 1) {
            $check_flag = true;
            break;
          } elseif ($check->fields['zone_id'] == $order->billing['zone_id']) {
            $check_flag = true;
            break;
          }
          $check->MoveNext();
        }

        if ($check_flag == false) {
          $this->enabled = false;
        }
      }
    }

    function javascript_validation() {
      return false;
    }

    function selection() {
      return array('id' => $this->code,
                   'module' => $this->title);
    }

    function pre_confirmation_check() {
      return false;
    }

    function confirmation() {
      return false;
    }

    function process_button() {
      global $order;

      $ccfields = '<div class="tcbpay-ccwrapper">'."\n";
      $ccfields .= '<div>'."\n";
      $ccfields .= '  <label class="inputLabel" for="cardholder">'.MODULE_PAYMENT_TCBPAY_ENTRY_CARDHOLDER.'</label>'."\n";
      $ccfields .= zen_draw_input_field('cardholder', $order->billing['firstname'] . ' ' . $order->billing['lastname'], 'id="cardholder"')."\n";
      $ccfields .= '</div>'."\n";
      $ccfields .= '<div>'."\n";
      $ccfields .= '  <label class="inputLabel" for="cardnumber">'.MODULE_PAYMENT_TCBPAY_ENTRY_CARDNUMBER.'</label>'."\n";
      $ccfields .= zen_draw_input_field('cardnumber', '', 'id="cardnumber"').zen_draw_hidden_field('enccardnum', '', 'id="enccardnum"')."\n";
      $ccfields .= '</div>'."\n";


      for ($i=1; $i<13; $i++) {
        $expires_month[] = array('id' => sprintf('%02d', $i), 'text' => strftime('%B',mktime(0,0,0,$i,1,2000)));
      }

      $today = getdate(); 
      for ($i=$today['year']; $i < $today['year']+10; $i++) {
        $expires_year[] = array('id' => strftime('%y',mktime(0,0,0,1,1,$i)), 'text' => strftime('%Y',mktime(0,0,0,1,1,$i)));
      }

      $ccfields .= '<div>'."\n";
      $ccfields .= '  <label class="inputLabel" for="expiry_month">'.MODULE_PAYMENT_TCBPAY_ENTRY_CARDEXPIRY.'</label>'."\n";
      $ccfields .= '<div>'."\n";
      $ccfields .= zen_draw_pull_down_menu('expiry_month', $expires_month, '', 'id="expiry_month"') ."\n";
      $ccfields .= zen_draw_pull_down_menu('expiry_year', $expires_year, 'id="expiry_year"') ."\n";
      $ccfields .= '</div>'."\n";
      $ccfields .= '</div>'."\n";
      if (MODULE_PAYMENT_TCBPAY_USECVV == 'True') {
        $ccfields .= '<div>'."\n";
        $ccfields .= '  <label class="inputLabel" for="cardcvv">'.MODULE_PAYMENT_TCBPAY_ENTRY_CVV.'</label>'."\n";
        $ccfields .= zen_draw_input_field('cardcvv', '', 'id="cardcvv" maxlength="4"')."\n";
        $ccfields .= '</div>'."\n";
      }
      $ccfields .= '</div>'."\n";

      $api_js = 'https://gateway2.tcbpay.com/services/proxynization_api.js';
      if (MODULE_PAYMENT_TCBPAY_TESTMODE == 'True') {
        $api_js = 'https://gateway2-stg.tcbpay.com/services/proxynization_api.js';
      }
      $javascript = '<script type="text/javascript" src="'.$api_js.'"></script>';
      $javascript .= "<script>\n" .
      "$(document).ready(function() {\n" .
      "  $('form[name=\"checkout_confirmation\"] .button_confirm_order').on('click', function(e) {\n" .
      "    $(this).prop('disabled', true);\n" .
      "    $('#tcbpay-cardform .error').remove();\n" .
      "    var emptyFields = $('[name=\"checkout_confirmation\"] input[type=\"text\"]').filter(function(index, elem) {\n" .
      "      return $(elem).attr('name').match(/^card/) && $.trim($(elem).val()) === '';\n" .
      "    });\n" .
      "    emptyFields.first().focus();\n" .
      "    if (emptyFields.length > 0) {\n" .
      "      $(this).prop('disabled', false);\n" .
      "    } else {\n" .
      "      TCBPayProxynizationAPI.process('#cardnumber', 'tcbpayClientCallback');\n" .
      "    }\n" .
      "    return false;\n" .
      "  });\n" .
      "});\n" .
      "function tcbpayClientCallback(respCode, respMsg, proxyCard) {\n" .
      "  if (respCode == \"A01\") {\n" .
      "    $('#enccardnum').val(proxyCard);\n" .
      "    $('#cardnumber').val('');\n" .
      "    $('form[name=\"checkout_confirmation\"]').submit();\n" .
      "  } else {\n" .
      "    $('[name=\"checkout_confirmation\"]').append('<div class=\"error text-danger text-center\">'+respMsg+'</div>');\n" .
      "  }\n" .
      "}\n" .
      "</script>\n";
      $style = "<style>.tcbpay-ccwrapper{margin-top:20px;max-width:400px;} .tcbpay-ccwrapper > div > div select {display: inline-block; width: 25%;} .tcbpay-ccwrapper > div input, .tcbpay-ccwrapper > div select { height: 35px; border: 1px solid #ccc; margin-bottom: 10px;}.tcbpay-ccwrapper > div input#cardcvv{width: 70px;}</style>";

      return $ccfields . $javascript . $style;
    }

    function before_process() {
      global $order, $messageStack;

      include(DIR_WS_CLASSES . 'tcbpay.php');
      $tcbpay = new Tcbpay_Api;
      $ccNumber = '';
      $encCardNum = trim($_POST['enccardnum']);
      $ccExpMonth = trim($_POST['expiry_month']);
      $ccExpYear = trim($_POST['expiry_year']);
      $cardCVV = '';
      if (MODULE_PAYMENT_TCBPAY_USECVV == 'True') {
        $cardCVV = trim($_POST['cardcvv']);
      }

      $error = '';
      if (!trim($_POST['cardholder'])) {
        $error = MODULE_PAYMENT_TCBPAY_ERR_RCARDHOLDER;
      } elseif (!$encCardNum) {
        $error = MODULE_PAYMENT_TCBPAY_ERR_RCARDNUMBER;
      } elseif ($encCardNum) {
        $response = $tcbpay->decodeCard($encCardNum);
        $dcresp = $response['resp'];
        if ($response['success']) {
          $ccNumber = preg_replace('/[\-\s]+/', '', trim($dcresp->accountNumber));
        } elseif ($dcresp) {
          $error = $dcresp->description;
        } else {
          $error = 'TCBPay gateway error.';
        }
      } elseif (MODULE_PAYMENT_TCBPAY_USECVV == 'True' && !$cardCVV) {
        $error = MODULE_PAYMENT_TCBPAY_ERR_RCARDCVV;
      }

      if (!$error) {
        $cc_validation = $tcbpay->card_validation();
        $cvld = $cc_validation->validate($ccNumber, $ccExpMonth, $ccExpYear);

        switch (true) {
          case $cvld === -1:
            $error = sprintf(MODULE_PAYMENT_TCBPAY_ERR_CCACCEPT, substr($cc_validation->cc_number, 0, 4));
            break;
          case $cvld === -2:
          case $cvld === -3:
          case $cvld === -4:
            $error = MODULE_PAYMENT_TCBPAY_ERR_EXPIRY;
            break;
          case $cvld === -5:
          case $cvld === -6:
          case $cvld === -7:
            $error = 'cvv';
            break;
          case $cvld === -8:
            $error = MODULE_PAYMENT_TCBPAY_ERR_CCNOTACCEPT;
            break;
          case $cvld === false:
            $error = MODULE_PAYMENT_TCBPAY_ERR_CCINVALID;
            break;
        }

        $cctypes = explode(',', MODULE_PAYMENT_TCBPAY_CCTYPES);
        foreach($cctypes as $cctpe) {
          $cctypes[] = trim($cctpe);
        }

        if (!in_array($cc_validation->cc_type, $cctypes)) {
          $error = MODULE_PAYMENT_TCBPAY_ERR_CCNOTACCEPT;
        }

        if (MODULE_PAYMENT_TCBPAY_USECVV == 'True' && !preg_match($cc_validation->getCvvRegEx()[$cc_validation->cc_type], $cardCVV)) {
          $error = MODULE_PAYMENT_TCBPAY_ERR_CVV;
        }

        if (!$error) {
          $zone_code = zen_get_zone_code($order->billing['country_id'], $order->billing['zone_id'], '');

          $post_data = array(
              "amount"            => round($order->info['total'], 2),
              "creditcard_number" => $encCardNum,                
              "expire_month"      => str_pad($ccExpMonth, 2, 0, STR_PAD_LEFT),
              "expire_year"       => $ccExpYear,
              "cvv"               => $cardCVV,
              "first_name"        => $order->billing['firstname'],
              "last_name"         => $order->billing['lastname'],
              "address"           => $order->billing['street_address'],
              "city"              => $order->billing['city'],
              "state"             => $zone_code,
              "zip_code"          => $order->billing['postcode'],
              "country_code"      => $order->billing['country']['iso_code_2'],
              "phone_number"      => $order->customer['telephone'],
              "email"             => $order->customer['email_address'],
              "currency"          => $order->info['currency'],
              "memo"              => $order->info['comments']
          );

          $response = $tcbpay->sendRequest($post_data);
          $resp = $response['resp'];

          if ($response['success']) {
            $this->transaction_id = $resp->transaction_id;
          } elseif ($resp) {
            $error = $resp->description;
          } else {
            $error = 'TCBPay gateway error.';
          }
        }
      }

      if ( $error ) {
        $messageStack->add_session('checkout_payment', $error, 'error');
        zen_redirect(zen_href_link(FILENAME_CHECKOUT_PAYMENT, '', 'SSL', true, false));
      }
    }

    function after_process() {
      global $insert_id;

      if ($this->transaction_id) {
        $comments = 'Transaction Id: ' . $this->transaction_id;
        zen_update_orders_history($insert_id, $comments, null, $this->order_status, -1);
      }
    }

    function get_error() {
      $error = array('title' => MODULE_PAYMENT_TCBPAY_ERROR_TITLE,
                     'error' => stripslashes(urldecode($_GET['error'])));

      return $error;
    }

    function check() {
      global $db;

      if (!isset($this->_check)) {
        $check_query = $db->Execute("select configuration_value from " . TABLE_CONFIGURATION . " where configuration_key = 'MODULE_PAYMENT_TCBPAY_STATUS'");
        $this->_check = $check_query->RecordCount();
      }

      return $this->_check;
    }

    function install() {
      global $db, $messageStack;

      if (defined('MODULE_PAYMENT_TCBPAY_STATUS')) {
        $messageStack->add_session('TCBPay module already installed.', 'error');
        zen_redirect(zen_href_link(FILENAME_MODULES, 'set=payment&module=tcbpay', 'NONSSL'));
        return 'failed';
      }

      $db->Execute("insert into " . TABLE_CONFIGURATION . " (configuration_title, configuration_key, configuration_value, configuration_description, configuration_group_id, sort_order, set_function, date_added) values ('Enable TCB Pay Module', 'MODULE_PAYMENT_TCBPAY_STATUS', 'True', 'Do you want to accept TCB Pay payments?', '6', '1', 'zen_cfg_select_option(array(\'True\', \'False\'), ', now())");
      $db->Execute("insert into " . TABLE_CONFIGURATION . " (configuration_title, configuration_key, configuration_value, configuration_description, configuration_group_id, sort_order, date_added) values ('Username', 'MODULE_PAYMENT_TCBPAY_USERNAME', '', 'TCB Pay merchant username', '6', '2', now())");
      $db->Execute("insert into " . TABLE_CONFIGURATION . " (configuration_title, configuration_key, configuration_value, configuration_description, configuration_group_id, sort_order, date_added) values ('Api Key', 'MODULE_PAYMENT_TCBPAY_PASSWORD', '', 'TCB Pay merchant api key', '6', '3', now())");
      $db->Execute("insert into " . TABLE_CONFIGURATION . " (configuration_title, configuration_key, configuration_value, configuration_description, configuration_group_id, sort_order, date_added) values ('Secret Key', 'MODULE_PAYMENT_TCBPAY_SECRETKEY', '', 'TCB Pay merchant secret key', '6', '4', now())");
      $db->Execute("insert into " . TABLE_CONFIGURATION . " (configuration_title, configuration_key, configuration_value, configuration_description, configuration_group_id, sort_order, set_function, date_added) values ('Credit Card Types', 'MODULE_PAYMENT_TCBPAY_CCTYPES', 'AE,VI,MC,DI,JCB', 'Enable this payment method for card types.', '6', '5', 'zen_cfg_pull_down_tcbpay_cctypes(', now())");
      $db->Execute("insert into " . TABLE_CONFIGURATION . " (configuration_title, configuration_key, configuration_value, configuration_description, configuration_group_id, sort_order, set_function, date_added) values ('Enable CVV', 'MODULE_PAYMENT_TCBPAY_USECVV', 'True', 'Do you want to enable CVV field to customers?', '6', '6', 'zen_cfg_select_option(array(\'True\', \'False\'), ', now())");
      $db->Execute("insert into " . TABLE_CONFIGURATION . " (configuration_title, configuration_key, configuration_value, configuration_description, configuration_group_id, sort_order, set_function, date_added) values ('Test Mode', 'MODULE_PAYMENT_TCBPAY_TESTMODE', 'True', 'Is TCB Pay test mode?', '6', '7', 'zen_cfg_select_option(array(\'True\', \'False\'), ', now())");
      $db->Execute("insert into " . TABLE_CONFIGURATION . " (configuration_title, configuration_key, configuration_value, configuration_description, configuration_group_id, sort_order, set_function, use_function, date_added) values ('Set Order Status', 'MODULE_PAYMENT_TCBPAY_ORDER_STATUS_ID', '0', 'Set the status of orders made with this payment module to this value', '6', '8', 'zen_cfg_pull_down_order_statuses(', 'zen_get_order_status_name', now())");
      $db->Execute("insert into " . TABLE_CONFIGURATION . " (configuration_title, configuration_key, configuration_value, configuration_description, configuration_group_id, sort_order, use_function, set_function, date_added) values ('Payment Zone', 'MODULE_PAYMENT_TCBPAY_ZONE', '0', 'If a zone is selected, only enable this payment method for that zone.', '6', '9', 'zen_get_zone_class_title', 'zen_cfg_pull_down_zone_classes(', now())");
      $db->Execute("insert into " . TABLE_CONFIGURATION . " (configuration_title, configuration_key, configuration_value, configuration_description, configuration_group_id, sort_order, date_added) values ('Sort order of display.', 'MODULE_PAYMENT_TCBPAY_SORT_ORDER', '0', 'Sort order of display. Lowest is displayed first.', '6', '10', now())");
    }

    function remove() {
      global $db;

      $db->Execute("delete from " . TABLE_CONFIGURATION . " where configuration_key in ('" . implode("', '", $this->keys()) . "')");
    }

    function keys() {
      return array('MODULE_PAYMENT_TCBPAY_STATUS', 'MODULE_PAYMENT_TCBPAY_USERNAME', 'MODULE_PAYMENT_TCBPAY_PASSWORD', 'MODULE_PAYMENT_TCBPAY_SECRETKEY', 'MODULE_PAYMENT_TCBPAY_CCTYPES', 'MODULE_PAYMENT_TCBPAY_USECVV', 'MODULE_PAYMENT_TCBPAY_TESTMODE', 'MODULE_PAYMENT_TCBPAY_ZONE', 'MODULE_PAYMENT_TCBPAY_ORDER_STATUS_ID', 'MODULE_PAYMENT_TCBPAY_SORT_ORDER');
    }
  }
